/*******************************************************************************
* POP-BOT V1.0    
* Filename : VRBot.pde
* POP-BOT-VRBot control program
********************************************************************************/

#include <SoftwareSerial.h>
#include "protocol.h"

#define LCD_CLEAR 0x01
#define LCD_LINE_1 0x80
#define LCD_LINE_2 0xC0

SoftwareSerial LCD_Serial = SoftwareSerial(16,16);

uint8_t _receivePin;
uint8_t _transmitPin;
long _baudRate;
int _bitPeriod;

void setup(){  

  // PC UART Init
  pinMode(0, INPUT);         // sets the digital pin as output
  pinMode(1, OUTPUT);        // sets the digital pin as output

  // VRbot UART Init
  pinMode(12, INPUT);        // sets the digital pin as output
  pinMode(13, OUTPUT);       // sets the digital pin as output

  // Motor Init
  pinMode(3,OUTPUT);        // Motor A1
  pinMode(5,OUTPUT);        // Motor A2
  pinMode(6,OUTPUT);        // Motor B2
  pinMode(9,OUTPUT);        // Motor B1
  
  // Button, LED and Speaker Init
  pinMode(2,INPUT);         // Setup Button pin Di2 ==> Input
  pinMode(14,OUTPUT);       // Setup Speaker pin Di14 ==> Output  
  pinMode(4,OUTPUT);        // Setup LED pin Di4 ==> Output  
  digitalWrite(4,HIGH);

  // LCD Init
  digitalWrite(16,HIGH);
  delay(500);
  pinMode(16,OUTPUT);
  LCD_Serial.begin(9600);  
  
  LCD_Clear();
  
  // if button 2 (Di2) is pressed enter in bride mode
  // for communication with VRbot GUI software 
  
  if (digitalRead(2)==0){
    beep();
    LCD_Print(LCD_LINE_1,"VRbot");
    LCD_Print(LCD_LINE_2,"Bridge mode");
    while(1)
    {
      int pc2vr = digitalRead(0);
      digitalWrite(13, pc2vr);
    
      int vr2pc = digitalRead(12);
      digitalWrite(1, vr2pc);
    }
  }
  
  VRbot_setup();
  
  if (!VRbot_Detect())                 
    LCD_Print(LCD_LINE_1, "VRbot NA");
  else {
    LCD_Print(LCD_LINE_1, "VRbot detected");
    LCD_Print(LCD_LINE_2, "Lang: English");
    VRbot_SetLanguage(0);  
    delay(1500);
  }  

}

void loop(){ 
  
  // implement speraker independent recognition 
  SI_Recognition(); 
  // or 
  // SD_Recognition();   
  // according to your need
}

void SI_Recognition()
{
  int cmd;
  
  LCD_Clear();
    
  LCD_Print(LCD_LINE_1,"Say Trigger!");
  digitalWrite(4,LOW); // LED ON  
  
  VRbot_RecognizeSI(0);      // recognize trigger word
  cmd = VRbot_CheckResult(); 
  
  beep();  
  digitalWrite(4,HIGH); // LED OFF
  
  LCD_Clear();
  
  LCD_Print(LCD_LINE_1,"Wordset 1");
  LCD_Print(LCD_LINE_2,"Say a command!");
  digitalWrite(4,LOW);  // LED ON

  VRbot_RecognizeSI(1);        // start SI recognition wordset 1
  cmd = VRbot_CheckResult();   // check recognition result  
  
  digitalWrite(4,HIGH); // LED OFF

  switch (cmd){
    
    case 1: // MOVE
      LCD_Print(LCD_LINE_1,"Commmand: MOVE");
      delay (200);
      
      LCD_Print(LCD_LINE_2,"Say a command!");
      digitalWrite(4,LOW);
      beep();
      
      VRbot_RecognizeSI(2);       // start SI recognition wordset 2
      cmd = VRbot_CheckResult();  // check recognition result

      digitalWrite(4,HIGH);
      
      switch (cmd){
        case 4:  // FORWARD
          LCD_Print(LCD_LINE_2,"Commmand: FWD");
          Forward(100);  // move robot forward
          delay(1000);
          Stop();
          break;

        case 5:  // BACWARD
          LCD_Print(LCD_LINE_2,"Commmand: BWD");
          Backward(100); // move robot backward
          delay(1000);
          Stop();          
          break;

        default: // ERROR or other commands
          LCD_Print(LCD_LINE_2,"Commmand: error!");
          delay(1000);
          break;
      }
      
      break;
      
    case 2:  // TURN
      LCD_Print(LCD_LINE_1,"Commmand: TURN");
      delay (200);
      
      LCD_Print(LCD_LINE_2,"Say a command!");
      digitalWrite(4,LOW);
      beep();
      
      VRbot_RecognizeSI(2);       // start SI recognition wordset 2     
      cmd = VRbot_CheckResult();  // check recognition result
      
      digitalWrite(4,HIGH);
      
      switch (cmd){
        case 0: // LEFT
          LCD_Print(LCD_LINE_2,"Commmand: LEFT");
          Spin_Left(100);  // move robot left
          delay(1000);
          Stop();
          break;
        case 1:
          LCD_Print(LCD_LINE_2,"Commmand: RIGHT");
          Spin_Right(100);   // move robot right
          delay(1000);
          Stop();          
          break;
        default: // ERROR or other commands
          LCD_Print(LCD_LINE_2,"Commmand: error!");
          delay(1000);
          break;
         
      }
      
      break;
    
    default: // Error or Other commands
      delay(200);
      break;
  
  }
  
}


void SD_Recognition()
{
  int cmd;
  
  LCD_Clear();
    
  LCD_Print(LCD_LINE_1,"Say Trigger!");
  digitalWrite(4,LOW); // LED ON
  
  VRbot_RecognizeSD(0);      // start SI trigger word recognition
  cmd = VRbot_CheckResult(); // check recognition result
  
  beep();  
  digitalWrite(4,HIGH); // LED OFF
  
  LCD_Clear();
  
  LCD_Print(LCD_LINE_1,"Group 1");
  LCD_Print(LCD_LINE_2,"Say a command!");
  digitalWrite(4,LOW);  // LED ON

  VRbot_RecognizeSD(1);        // start SD recognition group 1
  cmd = VRbot_CheckResult();   // check recognition result
  
  digitalWrite(4,HIGH); // LED OFF

  switch (cmd){
    
    case 0: // USER SD WORD 0
      LCD_Print(LCD_LINE_2,"Commmand: word 0");
      Dance();
      break;

    case 1: // USER SD WORD 1
      LCD_Print(LCD_LINE_2,"Commmand: word 1");
      Sing();      
      break;
  }
  
}


/*******************************************************************************/

void Stop(){    
  analogWrite(3,LOW);
  digitalWrite(5,LOW);
  analogWrite(6,LOW); 
  digitalWrite(9,LOW);
}
void Forward(int speed){    
  analogWrite(3,speed);
  digitalWrite(5,LOW);
  analogWrite(6,speed); 
  digitalWrite(9,LOW);
}
void Backward(int speed){    
  analogWrite(5,speed);
  digitalWrite(3,LOW);
  analogWrite(9,speed); 
  digitalWrite(6,LOW);
}
void Spin_Left(int speed){
  analogWrite(5,speed);
  digitalWrite(3,LOW);
  analogWrite(6,speed); 
  digitalWrite(9,LOW);
}
void Spin_Right(int speed){
  analogWrite(3,speed);
  digitalWrite(5,LOW);
  analogWrite(9,speed); 
  digitalWrite(6,LOW);
}

/*******************************************************************************/
void Dance(){

  for (int i = 0; i < 3; i++) {
    Spin_Left(100);
    delay(400);
    Spin_Right(100);
    delay(800);
    Forward(100);
    delay(400);
  }
  Stop();
  
}

void Sing(){
  int length = 15; // the number of notes
  char notes[] = "ccggaagffeeddc "; // a space represents a rest
  int beats[] = { 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 2, 4 };
  int tempo = 300;
  
  for (int i = 0; i < length; i++) {
    if (notes[i] == ' ') {
       delay(beats[i] * tempo); // rest
     } 
     else {
       playNote(notes[i], beats[i] * tempo);
     }
   // pause between notes
   delay(tempo / 2); 
  }  
}


/*******************************************************************************/

void LCD_Clear(){
  LCD_Serial.print(0xFE,BYTE);        // Command
  LCD_Serial.print(LCD_CLEAR,BYTE);   // Clear Screen Command
}
void LCD_Print(int line, char* text){
  LCD_Serial.print(0xFE,BYTE);        // Command
  LCD_Serial.print(line,BYTE);        // Line  
  LCD_Serial.print(text);             // Text
}

/*******************************************************************************/

void beep(){
  unsigned long us;
  int duration_,i;
  int duration = 25000;
  int freq = 3000;
  us=(1000000/(freq*2));
  duration_ = (duration/(us*2));
  for (i=0;i<duration_;i++){
    digitalWrite(14,HIGH);
    delayMicroseconds(us);
    digitalWrite(14,LOW);
    delayMicroseconds(us);
  } 
}

void playTone(int tone, int duration) {
  for (long i = 0; i < duration * 1000L; i += tone * 2) {
    digitalWrite(14, HIGH);
    delayMicroseconds(tone);
    digitalWrite(14, LOW);
    delayMicroseconds(tone);
  }
}

void playNote(char note, int duration) {
  char names[] = { 'c', 'd', 'e', 'f', 'g', 'a', 'b', 'C' };
  int tones[] = { 1915, 1700, 1519, 1432, 1275, 1136, 1014, 956 };
  
  // play the tone corresponding to the note name
  for (int i = 0; i < 8; i++) {
    if (names[i] == note) {
      playTone(tones[i], duration);
    }
  }
}


/*******************************************************************************/

void VRbot_setup()
{
  _baudRate = 9600;
  _bitPeriod = 1000000 / _baudRate;
  _receivePin = 12;
  _transmitPin = 13;
  digitalWrite(_transmitPin, HIGH);
  delayMicroseconds( _bitPeriod); 
}

unsigned char VRbot_read(void)
{
  uint8_t val = 0;
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(100);
  
  // one byte of serial data (LSB first)
  // ...--\    /--\/--\/--\/--\/--\/--\/--\/--\/--...
  //	 \--/\--/\--/\--/\--/\--/\--/\--/\--/
  //	start  0   1   2   3   4   5   6   7 stop

  while (digitalRead(_receivePin));

  // confirm that this is a real start bit, not line noise
  if (digitalRead(_receivePin) == LOW) {
    // frame start indicated by a falling edge and low start bit
    // jump to the middle of the low start bit
    delayMicroseconds(bitDelay / 2 - clockCyclesToMicroseconds(50));
	
    // offset of the bit in the byte: from 0 (LSB) to 7 (MSB)
    for (int offset = 0; offset < 8; offset++) {
	// jump to middle of next bit
	delayMicroseconds(bitDelay);
	
	// read bit
	val |= digitalRead(_receivePin) << offset;
    }
	
    delayMicroseconds(_bitPeriod);
    
    return val;
  }
  
  return -1;
}

void VRbot_write(uint8_t b)
{
  if (_baudRate == 0)
    return;
    
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(50); // a digitalWrite is about 50 cycles
  byte mask;

  digitalWrite(_transmitPin, LOW);
  delayMicroseconds(bitDelay);

  for (mask = 0x01; mask; mask <<= 1) {
    if (b & mask){ // choose bit
      digitalWrite(_transmitPin,HIGH); // send 1
    }
    else{
      digitalWrite(_transmitPin,LOW); // send 1
    }
    delayMicroseconds(bitDelay);
  }

  digitalWrite(_transmitPin, HIGH);
  delayMicroseconds(bitDelay);
}


unsigned char VRbot_Detect(void) {
  unsigned char i;
  for (i = 0; i < 5; ++i) {
    delay(100);
    VRbot_write(CMD_BREAK);
    if ( VRbot_read() == STS_SUCCESS)
      return 255;
  }
  return 0;
}

unsigned char VRbot_SetLanguage(unsigned char lang) {
        
  VRbot_write(CMD_LANGUAGE);
  delay(5);
  VRbot_write(ARG_ZERO + lang);

  if (VRbot_read() == STS_SUCCESS)
    return 255;
  return 0;
}

void VRbot_RecognizeSD(unsigned char group) {
  VRbot_write(CMD_RECOG_SD);
  delay(5);
  VRbot_write(ARG_ZERO + group);
}

void VRbot_RecognizeSI(unsigned char ws) {
  VRbot_write(CMD_RECOG_SI);
  delay(5);
  VRbot_write(ARG_ZERO + ws);
}

signed char VRbot_CheckResult(void) {
  
  unsigned char rx;
  rx = VRbot_read();
  if (rx == STS_SIMILAR || rx == STS_RESULT) {
    delay(5);
    VRbot_write(ARG_ACK);
    return (VRbot_read() - ARG_ZERO);
  }
  return -2;
}

unsigned char VRbot_GetGroupCount(unsigned char group) {
  unsigned char rx;
  VRbot_write(CMD_COUNT_SD);
  delay(5);
  VRbot_write(ARG_ZERO + group);
  if (VRbot_read() == STS_COUNT) {
    delay(5);
    VRbot_write(ARG_ACK);
    return VRbot_read() - ARG_ZERO;
  }
  return 0;
}



