unsigned short shifter, portd_index;
unsigned short portd_array[4];

void interrupt() {
	PORTD = 0;                             // Turn off all 7seg displays
	PORTA &= 0x0F;
	PORTA |= shifter;                      // turn on appropriate 7seg. display
	PORTD = portd_array[portd_index];      // bring appropriate value to PORTD

	// move shifter to next digit
	shifter <<= 1;
	// increment portd_index
	if (++portd_index & 4) {
		portd_index = 0;                     // turn on 1st, turn off 2nd 7seg.
		shifter = 1;
	}
	INTCON.T0IF = 0;
} //~

//-------------- Returns mask for common cathode 7-seg. display
unsigned short mask(unsigned short num) {
	switch (num) {
		case 0 :
		return 0x3F;
		case 1 :
		return 0x06;
		case 2 :
		return 0x5B;
		case 3 :
		return 0x4F;
		case 4 :
		return 0x66;
		case 5 :
		return 0x6D;
		case 6 :
		return 0x7D;
		case 7 :
		return 0x07;
		case 8 :
		return 0x7F;
		case 9 :
		return 0x6F;
	} //case end
} //~

void display_number(unsigned int number) {
	unsigned int digit;

	digit = number / 1000u ;             // extract thousands digit
	portd_array[3] = mask(digit);        // and store it to PORTD array
	digit = (number / 100u) % 10u;       // extract hundreds digit
	portd_array[2] = mask(digit);        // and store it to PORTD array
	digit = (number / 10u) % 10u;        // extract tens digit
	portd_array[1] = mask(digit);        // and store it to PORTD array
	digit = number % 10u;                // extract ones digit
	portd_array[0] = mask(digit);        // and store it to PORTD array
}

unsigned int i;

#include "VRbot.h"

void demo_sd(void);
void demo_si(void);

void main(void) {
	// Setup hardware UART: RC7=RX, RC6=TX
	// Connected to VRbot module (enable Pull-UP on RC7)
	Usart_Init(9600);

	// Hi-Z for direct routing to PC through RS232
	TRISB.F4 = 1;
	TRISB.F5 = 1;
	ANSELH.F4 = 0;
	ANSELH.F5 = 0;
	WPUB.F4 = 1;
	WPUB.F5 = 1;
	OPTION_REG.NOT_RBPU = 0;

	// Setup RE0-3 as button inputs
	PORTE &= 0xF0;
	TRISE |= 0x0F;
	ANSEL &= 0x0F;

	// Setup 7-Segment Display
	PORTA &= 0xF0;
	TRISA &= 0xF0;              // Set PORTA as output
	PORTD = 0;
	TRISD = 0;                  // Set PORTD as output

	asm { CLRWDT };             // Reset WDT and Prescaler
	OPTION_REG.T0CS = 0;        // Select Fosc/4 for Timer0
	OPTION_REG.PSA = 0;         // Assign Prescaler to Timer0
	OPTION_REG &= 0xF8;
	OPTION_REG |= 4;            // Set Prescaler 1:16 -> ~60Hz Refresh
	TMR0 = 0;
	INTCON.T0IE = 1;            // Enable T0IE

	// Initialize Timer0 variables
	portd_index = 0;
	shifter = 1;
	portd_array[3] = 0x00;      // ' '
	portd_array[2] = 0x00;      // ' '
	portd_array[1] = 0x00;      // ' '
	portd_array[0] = 0x00;      // ' '
	INTCON.GIE = 1;             // Enable GIE

	// Detect VRbot module
	if (VRbot_Detect())
		goto demo;

bridge:
	// Wait forever
	for (i = 0; ; ++i) {
		Delay_50us();
		if (i & 0x1000) {
			portd_array[3] = 0x40;      // '-'
			portd_array[2] = 0x73;      // 'P'
			portd_array[1] = 0x39;      // 'C'
			portd_array[0] = 0x40;      // '-'
		} else {
			portd_array[3] = 0x00;      // ' '
			portd_array[0] = 0x00;      // ' '
		}
	}

demo:
	// Wait for Demo selection
	for (i = 0; ; ++i) {
		if (i & 0x0800)
			portd_array[0] = 0x08;      // '_'
		else
			portd_array[0] = 0x00;      // ' '
		Delay_50us();
		if (i & 0x4000) {
			portd_array[3] = 0x6D;      // 'S'
			portd_array[2] = 0x5E;      // 'd'
			portd_array[1] = 0x00;      // ' '

			if (Button(&PORTE, 0, 10, 1))
				demo_sd();
		} else {
			portd_array[3] = 0x6D;      // 'S'
			portd_array[2] = 0x30;      // 'I'
			portd_array[1] = 0x00;      // ' '

			if (Button(&PORTE, 0, 10, 1))
				demo_si();
		}
	}
}

void demo_sd(void) {
	unsigned char group = 0;
	signed char res;

loop_sd:
	portd_array[3] = mask(group / 10);
	portd_array[2] = mask(group % 10);
	portd_array[1] = 0x00;      // ' '
	portd_array[0] = 0x00;      // ' '
	i = 0;
	while (VRbot_GetGroupCount(group) == 0) {
		if (++group > 16)
			group = 0;
		if (++i > 16) {
			portd_array[3] = 0x54;      // 'n'
			portd_array[2] = 0x5C;      // 'o'
			portd_array[1] = 0x6D;      // 'S'
			portd_array[0] = 0x5E;      // 'd'
			portd_array[2] |= 0x80;    // DOT on
			while (1)
				;
		}
		portd_array[3] = mask(group / 10);
		portd_array[2] = mask(group % 10);
	}
	while (Button(&PORTE, 0, 10, 1))
		;
wait_sd:
	portd_array[3] = mask(group / 10);
	portd_array[2] = mask(group % 10);
	portd_array[1] = 0x00;      // ' '
	portd_array[0] = 0x00;      // ' '
	VRbot_RecognizeSD(group);
	portd_array[2] |= 0x80;    // DOT on
	// Wait for result
	for (i = 0; ; ++i) {
		if (i & 0x0800) {
			portd_array[1] = 0x08;      // '_'
			portd_array[0] = 0x08;      // '_'
		} else {
			portd_array[1] = 0x00;      // '_'
			portd_array[0] = 0x00;      // ' '
		}
		Delay_50us();

		if (Button(&PORTE, 0, 10, 1)) {
			portd_array[2] &= 0x7F;  // DOT off
			VRbot_Detect();
			if (++group > 16)
				group = 0;
			goto loop_sd;
		}

		res = VRbot_CheckResult();
		if (res < -1)
			goto wait_sd;
		if (res >= 0) {
			portd_array[1] = mask(res / 10);
			portd_array[0] = mask(res % 10);
			Delay_ms(2000);
			goto wait_sd;
		}
	}
}

void demo_si(void) {
	unsigned char lang = 0, ws = 0;
	signed char res;

	while (Button(&PORTE, 0, 10, 1))
		;
	for (i = 0; ; ++i) {
		if (i & 0x0800)
			portd_array[0] = 0x08;      // '_'
		else
			portd_array[0] = 0x00;      // ' '
		Delay_50us();

		lang = (i & 0xC000) >> 14;
		switch (lang) {
			case 0:
			portd_array[3] = 0x79;      // 'E'
			portd_array[2] = 0x54;      // 'n'
			portd_array[1] = 0x6F;      // 'g'
			break;

			case 1:
			portd_array[3] = 0x30;      // 'I'
			portd_array[2] = 0x78;      // 't'
			portd_array[1] = 0x5F;      // 'a'
			break;

			case 2:
			portd_array[3] = 0x0E;      // 'J'
			portd_array[2] = 0x5F;      // 'a'
			portd_array[1] = 0x73;      // 'p'
			break;

			case 3:
			portd_array[3] = 0x3D;      // 'G'
			portd_array[2] = 0x7B;      // 'e'
			portd_array[1] = 0x50;      // 'r'
			break;
		}
		if (Button(&PORTE, 0, 10, 1)) {
			VRbot_SetLanguage(lang);
			break;
		}
	}

loop_si:
	portd_array[3] = 0x00;
	portd_array[2] = mask(ws);
	portd_array[1] = 0x00;      // ' '
	portd_array[0] = 0x00;      // ' '
	while (Button(&PORTE, 0, 10, 1))
		;
	VRbot_RecognizeSI(ws);
	portd_array[2] |= 0x80;    // DOT on
	// Wait for result
	for (i = 0; ; ++i) {
		if (i & 0x0800) {
			portd_array[1] = 0x08;      // '_'
			portd_array[0] = 0x08;      // '_'
		} else {
			portd_array[1] = 0x00;      // '_'
			portd_array[0] = 0x00;      // ' '
		}
		Delay_50us();

		if (Button(&PORTE, 0, 10, 1)) {
			portd_array[2] &= 0x7F;  // DOT off
			VRbot_Detect();
			if (++ws > 3)
				ws = 0;
			goto loop_si;
		}

		res = VRbot_CheckResult();
		if (res < -1)
			goto loop_si;
		if (res >= 0) {
			portd_array[1] = mask(res / 10);
			portd_array[0] = mask(res % 10);
			Delay_ms(2000);
			goto loop_si;
		}
	}
}
